# -*- coding: utf-8 -*-
import subprocess
import sys, os
import signal
import socket
import datetime, time
import smtplib
from email.mime.text import MIMEText
import argparse
from tempfile import gettempdir
import hashlib
import json
from gelfclient import UdpClient

def main():
    cli_parser = argparse.ArgumentParser()
    cli_parser.add_argument('-r', '--recipients', nargs='+', type=str, default=None, help='list of emails')
    cli_parser.add_argument('--comment', type=str, default=None, help='Comment for email message')
    cli_parser.add_argument('-c', '--command', type=str, required=True, help='Shell command')
    cli_parser.add_argument('--valid-exitcodes', type=int, nargs='+', default=[0, ], help='Valid exitcodes for executed command')
    cli_parser.add_argument('-d', '--daemon', default=False, action='store_true', help='Daemonize process after start')
    cli_parser.add_argument('--dont-duplicate', default=False, action='store_true', help='Not run process when process with same name already run')
    cli_parser.add_argument('--retry', type=int, default=1, help='Retry run N times on fail')
    cli_parser.add_argument('--retry-sleep', type=int, default=0, help='Sleep between retries (seconds)')
    cli_parser.add_argument('--config', type=str, default='/etc/ewn.conf', help='Config file')
    cli = cli_parser.parse_args()

    config = json.loads(open(cli.config, 'r').read())
    if cli.recipients is None:
        cli.recipients = config['email']['recipients']

    def popen(command):
        process = subprocess.Popen(str(command), stderr=subprocess.STDOUT, stdout=subprocess.PIPE, shell=True, executable='/bin/bash')
        output = process.communicate()[0]
        return process.returncode, output

    def sighup_handler(signal, frame):
        pass

    class RunFile():
        def __init__(self):
            self.file_name = hashlib.sha1(cli.command).hexdigest()
            self.tmp_file_path = '{0}/ewn-{1}.lock'.format(gettempdir(), self.file_name)
        
        def create(self):
            tfile = open(self.tmp_file_path, 'w')
            print(os.getpid())
            tfile.write(str(os.getpid()))
            tfile.close()

        def remove(self):
            try:
                os.remove(self.tmp_file_path)
            except OSError:
                pass

        def lock_exist(self):
            if (os.path.isfile(self.tmp_file_path)):
                pid = open(self.tmp_file_path, 'r').read()
                if (self.__pid_exist(pid)):
                    return True
            return False
        
        @staticmethod
        def __pid_exist(pid):
            if os.path.isdir('/proc/{}'.format(pid)):
                return True
            return False

    def send_to_email(subject, message):
        msg = MIMEText(message)
        msg['Subject'] = subject
        if ('from' in config['email']) and (config['email']['from'] is not None):
            msg['From'] = config['email']['from']
        else:
            msg['From'] = 'ewn@' + socket.gethostname()
        msg['To'] = ", ".join(cli.recipients)
        s = smtplib.SMTP(config['email']['host'])
        s.login(str(config['email']['user']), str(config['email']['pass']))
        s.sendmail(msg['From'], cli.recipients, msg.as_string())
        s.quit()


    def send_to_zabbix(status):
        command = 'zabbix_sender -c {0} -k "{1}" -o "{2}"'.format(config['zabbix']['config'], config['zabbix']['key'], status)
        exitcode, output = popen(command)
        if exitcode != 0:
            raise RuntimeError('Zabbix sender failed with error: {0}'.format(output))
        return

    if cli.daemon:
        pid=os.fork()
        if pid != 0:
            print('Process successfully daemonized. Pid: {0}'.format(pid))
            exit(0)
        else:
            sys.stdout.close()
            sys.stderr.close()
            sys.stdin.close()
            signal.signal(signal.SIGHUP, sighup_handler)

    message = 'Command: {0}\nHost: {1}\nComment: {2}\nValid exit codes: {3}\nRetries: {4}\nRetry sleep: {5}\nDaemonized: {6}\nPreserve duplicates: {7}'.format(
        cli.command, socket.gethostname(), cli.comment, cli.valid_exitcodes, cli.retry, cli.retry_sleep, cli.daemon, cli.dont_duplicate)
    exitcode = None

    if (cli.dont_duplicate is False) or (RunFile().lock_exist() is False):
        RunFile().create()
        for retry in range(1, cli.retry+1):
            if exitcode not in cli.valid_exitcodes:
                start_date = datetime.datetime.now()
                exitcode, output = popen(cli.command)
                finish_date = datetime.datetime.now()
                message += '\n\nRetry number: {0}\nStart date: {1}\nFinish date: {2}\nDuration: {3}\nExit code: {4}\nOutput:\n{5}'.format(
                    retry, str(start_date), str(finish_date), str(finish_date - start_date), exitcode, output)
                try:
                    graylog = UdpClient(config['graylog']['host'], port=config['graylog']['port'], mtu=config['graylog']['mtu'])
                    data = {}
                    data['short_message'] = cli.command
                    data['command'] = cli.command
                    data['start_date'] = str(start_date)
                    data['finish_date'] = str(finish_date)
                    data['duration'] = (finish_date - start_date).seconds
                    data['exitcode'] = exitcode
                    if len(output) > 32768:
                        data['output'] = output[0:32700] + '...(Elasticsearch limit exceeded)'
                    else:
                        data['output'] = output
                    data['comment'] = cli.comment
                    data['retry'] = "{0}/{1}".format(retry, cli.retry)
                    data['tag'] = config['graylog']['tag']
                    if exitcode not in cli.valid_exitcodes: data['failed'] = 1
                    else: data['failed'] = 0
                    if config['graylog']['enabled']:
                        graylog.log(data)
                except:
                    print('Failed to send info to Graylog.')
                if exitcode not in cli.valid_exitcodes:
                    time.sleep(cli.retry_sleep)
        RunFile().remove()
    else:
        print('Duplicate process found, skipping start')
        exitcode = 300
        output = 'Duplicate process found, skipping start'
        message += '\nOutput: {0}'.format(output)
        try:
            graylog = UdpClient(config['graylog']['host'], port=config['graylog']['port'], mtu=config['graylog']['mtu'])
            data = {}
            data['short_message'] = cli.command
            data['command'] = cli.command
            data['start_date'] = str(start_date)
            data['finish_date'] = str(finish_date)
            data['duration'] = (finish_date - start_date).seconds
            data['exitcode'] = exitcode
            if len(output) > 32768:
                data['output'] = output[0:32700] + '...(Elasticsearch limit exceeded)'
            else:
                data['output'] = output
            data['comment'] = cli.comment
            data['retry'] = "{0}/{1}".format(retry, cli.retry)
            data['tag'] = config['graylog']['tag']
            if exitcode not in cli.valid_exitcodes: data['failed'] = 1
            else: data['failed'] = 0
            if config['graylog']['enabled']:
                graylog.log(data)
        except:
            print('Failed to send info to Graylog.')
        
    if cli.daemon is False:
        print(message)
    if exitcode not in cli.valid_exitcodes: 
        if config['email']['enabled']:
            try: send_to_email(subject='ewn@{0} FAILED: {1} '.format(socket.gethostname(), cli.command), message=message)
            except: print("Failed to send email.")
        if config['zabbix']['enabled']:
            try: send_to_zabbix(1)
            except: print("Failed to send info to Zabbix.")
    else:
        if config['zabbix']['enabled']:
            try: send_to_zabbix(0)
            except: print("Failed to send info to Zabbix.")



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from shutil import copyfile
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))


setup(
    name='ewn',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='1.0.5',

    description='Execute With Notify',
    long_description="Whis is wrapper script, what can notify you when your tasks failed.",

    # The project's main homepage.
    url='https://github.com/larrabee/ewn',

    # Author details
    author='larrabee',
    author_email='kafatron@gmail.com',

    # Choose your license
    license='GPLv3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Logging',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='ewn wrapper',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['gelfclient==0.0.4'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={},

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={},

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'ewn=ewn:main',
        ],
    },
)

config_dir = '/etc/'
config_files = [
                   (config_dir, 'ewn.conf'),
               ]

for config in config_files:
    if not path.exists(config[0]):
        os.makedirs(config[0])
    if not path.exists(path.join(config[0], config[1])):
        copyfile(config[1], path.join(config[0], config[1]))
